@@ -3,13 +3,6 @@ import pytest
 from labgrid.exceptions import NoDriverFoundError
 
 
-@pytest.fixture()
-def strategy(target):
-    try:
-        return target.get_driver('BareboxStrategy')
-    except NoDriverFoundError:
-        pytest.skip("strategy not found")
-
 @pytest.fixture(scope="function")
 def in_bootloader(strategy, capsys):
     with capsys.disabled():
