@@ -47,7 +47,7 @@ class Handler(multiprocessing.Process):
         context['log'] = self.log.getChild('setup')
         setup = self._get_function('setup', self.context)
         if setup is None:
-            def setup():  # pylint: disable=function-redefined
+            def setup():
                 pass
         return setup
 
@@ -210,7 +210,7 @@ def main():
 
     env = Environment(config_file=args.config)
 
-    StepReporter()
+    StepReporter.start()
 
     manager = Manager(env, args)
     if not manager.configure():
