@@ -7,20 +7,23 @@ from .sshdriver import SSHDriver
 from .externalconsoledriver import ExternalConsoleDriver
 from .exception import CleanUpError, ExecutionError
 from .fastbootdriver import AndroidFastbootDriver
+from .dfudriver import DFUDriver
 from .openocddriver import OpenOCDDriver
 from .quartushpsdriver import QuartusHPSDriver
 from .flashromdriver import FlashromDriver
 from .onewiredriver import OneWirePIODriver
 from .powerdriver import ManualPowerDriver, ExternalPowerDriver, \
                          DigitalOutputPowerDriver, YKUSHPowerDriver, \
-                         USBPowerDriver, SiSPMPowerDriver
-from .usbloader import MXSUSBDriver, IMXUSBDriver, RKUSBDriver, UUUDriver
+                         USBPowerDriver, SiSPMPowerDriver, NetworkPowerDriver, \
+                         PDUDaemonDriver
+from .usbloader import MXSUSBDriver, IMXUSBDriver, BDIMXUSBDriver, RKUSBDriver, UUUDriver
 from .usbsdmuxdriver import USBSDMuxDriver
 from .usbsdwiredriver import USBSDWireDriver
 from .common import Driver
 from .qemudriver import QEMUDriver
 from .modbusdriver import ModbusCoilDriver
-from .sigrokdriver import SigrokDriver
+from .modbusrtudriver import ModbusRTUDriver
+from .sigrokdriver import SigrokDriver, SigrokPowerDriver
 from .usbstoragedriver import USBStorageDriver, NetworkUSBStorageDriver, Mode
 from .resetdriver import DigitalOutputResetDriver
 from .gpiodriver import GpioDigitalOutputDriver
@@ -34,7 +37,12 @@ from .pyvisadriver import PyVISADriver
 from .usbhidrelay import HIDRelayDriver
 from .flashscriptdriver import FlashScriptDriver
 from .usbaudiodriver import USBAudioInputDriver
+from .usbvideodriver import USBVideoDriver
+from .httpvideodriver import HTTPVideoDriver
 from .networkinterfacedriver import NetworkInterfaceDriver
-from .provider import TFTPProviderDriver
+from .provider import HTTPProviderDriver, NFSPProviderDriver, TFTPProviderDriver
 from .mqtt import TasmotaPowerDriver
 from .manualswitchdriver import ManualSwitchDriver
+from .usbtmcdriver import USBTMCDriver
+from .deditecrelaisdriver import DeditecRelaisDriver
+from .dediprogflashdriver import DediprogFlashDriver
