@@ -26,9 +26,8 @@ class ExternalConsoleDriver(ConsoleExpectMixin, Driver, ConsoleProtocol):
     def __attrs_post_init__(self):
         super().__attrs_post_init__()
         self.logger = logging.getLogger(f"{self}({self.target})")
-        self.status = 0  #pylint: disable=attribute-defined-outside-init
+        self.status = 0
         self._child = None
-        self.open()
 
     def open(self):
         """Starts the subprocess, does nothing if it is already closed"""
@@ -54,6 +53,7 @@ class ExternalConsoleDriver(ConsoleExpectMixin, Driver, ConsoleProtocol):
         if not self.status:
             return
         if self._child.poll() is not None:
+            self._child.communicate()
             raise ExecutionError("child has vanished")
         self._child.terminate()
         try:
@@ -70,17 +70,23 @@ class ExternalConsoleDriver(ConsoleExpectMixin, Driver, ConsoleProtocol):
         self._child = None
         self.status = 0
 
-    def _read(self, size: int = 1024, timeout: int = 0):
+    def _read(self, size: int = 1024, timeout: int = 0, max_size: int = None):
         """
         Reads 'size' bytes from the serialport
 
         Keyword Arguments:
         size -- amount of bytes to read, defaults to 1024
+        max_size -- maximal amount of bytes to read
         """
+        if max_size:
+            read_size = min(size, max_size)
+        else:
+            read_size = size
+
         if self._child.poll() is not None:
             raise ExecutionError("child has vanished")
         if self._poll.poll(timeout):
-            return self._child.stdout.read(size)
+            return self._child.stdout.read(read_size)
 
         return b''
 
@@ -97,5 +103,8 @@ class ExternalConsoleDriver(ConsoleExpectMixin, Driver, ConsoleProtocol):
         self._child.stdin.flush()
         return result
 
+    def on_activate(self):
+        self.open()
+
     def on_deactivate(self):
         self.close()
