@@ -1,4 +1,3 @@
-# pylint: disable=no-member
 import logging
 from itertools import chain
 import attr
@@ -63,18 +62,11 @@ class OpenOCDDriver(Driver, BootstrapProtocol):
                 self.search = [self.search]
 
     def _get_usb_path_cmd(self):
-        # OpenOCD supports "adapter usb location" since a1b308ab, if the command is not known
-        # notice user and continue
-        message = ("Your OpenOCD version does not support specifying USB paths.\n"
-                   "Consider updating to OpenOCD v0.11 when using multiple USB Blasters.")
-        return [
-            "--command",
-            "'if {{ [catch {{adapter usb location \"{usbpath}\"}}] }} {{ puts stderr \"{msg}\" }}'"
-            .format(usbpath=self.interface.path, msg=message)
-        ]
+        # OpenOCD supports "adapter usb location" since a1b308ab, released with 0.11.0-rc1
+        return ["--command", f"adapter usb location \"{self.interface.path}\""]
 
     def _run_commands(self, commands: list):
-        cmd = self.interface.command_prefix+[self.tool]
+        cmd = [self.tool]
         cmd += chain.from_iterable(("--search", path) for path in self.search)
         cmd += self._get_usb_path_cmd()
 
@@ -96,9 +88,9 @@ class OpenOCDDriver(Driver, BootstrapProtocol):
             cmd.append("--file")
             cmd.append(mconfig.get_remote_path())
 
-        cmd += chain.from_iterable(("--command", f"'{command}'") for command in commands)
+        cmd += chain.from_iterable(("--command", f"{command}") for command in commands)
         processwrapper.check_output(
-            cmd,
+            self.interface.wrap_command(cmd),
             print_on_silent_log=True
         )
 
