@@ -43,6 +43,11 @@ class QEMUDriver(ConsoleExpectMixin, Driver, PowerProtocol, ConsoleProtocol):
         rootfs (str): optional, reference to the paths key for use as the virtio-9p filesystem
         dtb (str): optional, reference to the image key for the device tree
         bios (str): optional, reference to the image key for the bios image
+        display (str, default="none"): optional, display output to enable; must be one of:
+            none: Do not create a display device
+            fb-headless: Create a headless framebuffer device
+            egl-headless: Create a headless GPU-backed graphics card. Requires host support
+        nic (str): optional, configuration string to pass to QEMU to create a network interface
     """
     qemu_bin = attr.ib(validator=attr.validators.instance_of(str))
     machine = attr.ib(validator=attr.validators.instance_of(str))
@@ -70,6 +75,16 @@ class QEMUDriver(ConsoleExpectMixin, Driver, PowerProtocol, ConsoleProtocol):
     bios = attr.ib(
         default=None,
         validator=attr.validators.optional(attr.validators.instance_of(str)))
+    display = attr.ib(
+        default="none",
+        validator=attr.validators.optional(attr.validators.and_(
+            attr.validators.instance_of(str),
+            attr.validators.in_(["none", "fb-headless", "egl-headless"]),
+        ))
+    )
+    nic = attr.ib(
+        default=None,
+        validator=attr.validators.optional(attr.validators.instance_of(str)))
 
     def __attrs_post_init__(self):
         super().__attrs_post_init__()
@@ -80,6 +95,7 @@ class QEMUDriver(ConsoleExpectMixin, Driver, PowerProtocol, ConsoleProtocol):
         self._tempdir = None
         self._socket = None
         self._clientsocket = None
+        self._forwarded_ports = {}
         atexit.register(self._atexit)
 
     def _atexit(self):
@@ -87,10 +103,10 @@ class QEMUDriver(ConsoleExpectMixin, Driver, PowerProtocol, ConsoleProtocol):
             return
         self._child.terminate()
         try:
-            self._child.wait(1.0)
+            self._child.communicate(timeout=1)
         except subprocess.TimeoutExpired:
             self._child.kill()
-            self._child.wait(1.0)
+            self._child.communicate(timeout=1)
 
     def on_activate(self):
         self._tempdir = tempfile.mkdtemp(prefix="labgrid-qemu-tmp-")
@@ -121,6 +137,11 @@ class QEMUDriver(ConsoleExpectMixin, Driver, PowerProtocol, ConsoleProtocol):
                 self._cmd.append(
                     f"if=sd,format={disk_format},file={disk_path},id=mmc0")
                 boot_args.append("root=/dev/mmcblk0p1 rootfstype=ext4 rootwait")
+            elif self.machine == "q35":
+                self._cmd.append("-drive")
+                self._cmd.append(
+                    f"if=virtio,format={disk_format},file={disk_path}")
+                boot_args.append("root=/dev/vda rootwait")
             elif self.machine == "pc":
                 self._cmd.append("-drive")
                 self._cmd.append(
@@ -163,12 +184,28 @@ class QEMUDriver(ConsoleExpectMixin, Driver, PowerProtocol, ConsoleProtocol):
         self._cmd.append(self.cpu)
         self._cmd.append("-m")
         self._cmd.append(self.memory)
-        self._cmd.append("-nographic")
+        if self.display == "none":
+            self._cmd.append("-nographic")
+        elif self.display == "fb-headless":
+            self._cmd.append("-display")
+            self._cmd.append("none")
+        elif self.display == "egl-headless":
+            self._cmd.append("-vga")
+            self._cmd.append("virtio")
+            self._cmd.append("-display")
+            self._cmd.append("egl-headless")
+        else:
+            raise ExecutionError(f"Unknown display '{self.display}'")
+
         self._cmd.append("-chardev")
         self._cmd.append(f"socket,id=serialsocket,path={sockpath}")
         self._cmd.append("-serial")
         self._cmd.append("chardev:serialsocket")
 
+        if self.nic:
+            self._cmd.append("-nic")
+            self._cmd.append(self.nic)
+
         if self.boot_args is not None:
             boot_args.append(self.boot_args)
         if self.kernel is not None and boot_args:
@@ -178,6 +215,11 @@ class QEMUDriver(ConsoleExpectMixin, Driver, PowerProtocol, ConsoleProtocol):
     def on_deactivate(self):
         if self.status:
             self.off()
+        if self._clientsocket:
+            self._clientsocket.close()
+            self._clientsocket = None
+        self._socket.close()
+        self._socket = None
         shutil.rmtree(self._tempdir)
 
     @step()
@@ -199,12 +241,18 @@ class QEMUDriver(ConsoleExpectMixin, Driver, PowerProtocol, ConsoleProtocol):
             self.qmp = QMPMonitor(self._child.stdout, self._child.stdin)
         except QMPError as exc:
             if self._child.poll() is not None:
+                self._child.communicate()
                 raise IOError(
                     f"QEMU process terminated with exit code {self._child.returncode}"
                 ) from exc
             raise
 
         self.status = 1
+
+        # Restore port forwards
+        for v in self._forwarded_ports.values():
+            self._add_port_forward(*v)
+
         self.monitor_command("cont")
 
     @step()
@@ -214,6 +262,7 @@ class QEMUDriver(ConsoleExpectMixin, Driver, PowerProtocol, ConsoleProtocol):
             return
         self.monitor_command('quit')
         if self._child.wait() != 0:
+            self._child.communicate()
             raise IOError
         self._child = None
         self.status = 0
@@ -223,21 +272,40 @@ class QEMUDriver(ConsoleExpectMixin, Driver, PowerProtocol, ConsoleProtocol):
         self.off()
         self.on()
 
-    @step(args=['command'])
-    def monitor_command(self, command):
+    @step(result=True, args=['command', 'arguments'])
+    def monitor_command(self, command, arguments={}):
         """Execute a monitor_command via the QMP"""
         if not self.status:
             raise ExecutionError(
                 "Can't use monitor command on non-running target")
-        return self.qmp.execute(command)
+        return self.qmp.execute(command, arguments)
 
-    def _read(self, size=1, timeout=10):
+    def _add_port_forward(self, proto, local_address, local_port, remote_address, remote_port):
+        self.monitor_command(
+            "human-monitor-command",
+            {"command-line": f"hostfwd_add {proto}:{local_address}:{local_port}-{remote_address}:{remote_port}"},
+        )
+
+    def add_port_forward(self, proto, local_address, local_port, remote_address, remote_port):
+        self._add_port_forward(proto, local_address, local_port, remote_address, remote_port)
+        self._forwarded_ports[(proto, local_address, local_port)] = (proto, local_address, local_port, remote_address, remote_port)
+
+    def remove_port_forward(self, proto, local_address, local_port):
+        del self._forwarded_ports[(proto, local_address, local_port)]
+        self.monitor_command(
+            "human-monitor-command",
+            {"command-line": f"hostfwd_remove {proto}:{local_address}:{local_port}"},
+        )
+
+    def _read(self, size=1, timeout=10, max_size=None):
         ready, _, _ = select.select([self._clientsocket], [], [], timeout)
         if ready:
             # Collect some more data
             time.sleep(0.01)
             # Always read a page, regardless of size
-            res = self._clientsocket.recv(4096)
+            size = 4096
+            size = min(max_size, size) if max_size else size
+            res = self._clientsocket.recv(size)
         else:
             raise TIMEOUT(f"Timeout of {timeout:.2f} seconds exceeded")
         return res
