@@ -31,7 +31,7 @@ class SerialDriver(ConsoleExpectMixin, Driver, ConsoleProtocol):
         message = ("The installed pyserial version does not contain important RFC2217 fixes.\n"
                    "You can install the labgrid fork via:\n"
                    "pip uninstall pyserial\n"
-                   "pip install https://github.com/labgrid-project/pyserial/archive/v3.4.0.1.zip#egg=pyserial\n")  # pylint: disable=line-too-long
+                   "pip install https://github.com/labgrid-project/pyserial/archive/v3.4.0.1.zip#egg=pyserial\n")
         warnings.warn(message)
 
     txdelay = attr.ib(default=0.0, validator=attr.validators.instance_of(float))
@@ -69,14 +69,33 @@ class SerialDriver(ConsoleExpectMixin, Driver, ConsoleProtocol):
     def on_deactivate(self):
         self.close()
 
-    def _read(self, size: int = 1, timeout: float = 0.0):
+    @Driver.check_bound
+    def get_export_vars(self):
+        export_vars = {
+            "speed": str(self.port.speed)
+        }
+        if isinstance(self.port, SerialPort):
+            export_vars["port"] = self.port.port
+        else:
+            host, port = proxymanager.get_host_and_port(self.port)
+            export_vars["host"] = host
+            export_vars["port"] = str(port)
+            export_vars["protocol"] = self.port.protocol
+        return export_vars
+
+    def _read(self, size: int = 1, timeout: float = 0.0, max_size: int = None):
         """
         Reads 'size' or more bytes from the serialport
 
         Keyword Arguments:
         size -- amount of bytes to read, defaults to 1
+        max_size -- maximal amount of bytes to read, values 'None' or '0' do not restrict the read
+                    length, defaults to None
+        if size == max_size: read and return exactly size = max_size bytes
         """
         reading = max(size, self.serial.in_waiting)
+        if max_size:  # limit reading to max_size if provided
+            reading = min(reading, max_size)
         self.serial.timeout = timeout
         res = self.serial.read(reading)
         if not res:
