@@ -1,10 +1,9 @@
-# pylint: disable=no-member,missing-kwoa,unused-argument
+# pylint: disable=unused-argument
 """The ShellDriver provides the CommandProtocol, ConsoleProtocol and
  InfoProtocol on top of a SerialPort."""
 import io
 import logging
 import re
-import time
 import shlex
 import ipaddress
 
@@ -61,9 +60,9 @@ class ShellDriver(CommandMixin, Driver, CommandProtocol, FileTransferProtocol):
         super().__attrs_post_init__()
         self.re_vt100 = re.compile(
             r'(\x1b\[|\x9b)[^@-_a-z]*[@-_a-z]|\x1b[@-_a-z]'
-        )  # pylint: disable=attribute-defined-outside-init,anomalous-backslash-in-string
+        )
         self.logger = logging.getLogger(f"{self}:{self.target}")
-        self._status = 0  # pylint: disable=attribute-defined-outside-init
+        self._status = 0
 
         self._xmodem_cached_rx_cmd = ""
         self._xmodem_cached_sx_cmd = ""
@@ -80,9 +79,6 @@ class ShellDriver(CommandMixin, Driver, CommandProtocol, FileTransferProtocol):
 
             self._put_ssh_key(keyfile_path)
 
-        # Turn off Kernel Messages to the console
-        self._run("dmesg -n 1")
-
     def on_deactivate(self):
         self._status = 0
 
@@ -134,6 +130,7 @@ class ShellDriver(CommandMixin, Driver, CommandProtocol, FileTransferProtocol):
         # occours, we can't lose any data this way.
         last_before = b''
         did_login = False
+        did_silence_kernel = False
 
         while True:
             index, before, _, _ = self.console.expect(
@@ -142,10 +139,15 @@ class ShellDriver(CommandMixin, Driver, CommandProtocol, FileTransferProtocol):
             )
 
             if index == 0:
-                # we got a promt. no need for any further action to activate
-                # this driver.
-                self.status = 1
-                break
+                if did_login and not did_silence_kernel:
+                    # Silence the kernel and wait for another prompt
+                    self.console.sendline("dmesg -n 1")
+                    did_silence_kernel = True
+                else:
+                    # we got a prompt. no need for any further action to
+                    # activate this driver.
+                    self.status = 1
+                    break
 
             elif index == 1:
                 # we need to login
@@ -200,7 +202,7 @@ class ShellDriver(CommandMixin, Driver, CommandProtocol, FileTransferProtocol):
         try:
             self.console.expect(
                 rf"{marker}\s+{self.prompt}",
-                timeout=5
+                timeout=30
             )
             self._status = 1
         except TIMEOUT:
@@ -217,7 +219,7 @@ class ShellDriver(CommandMixin, Driver, CommandProtocol, FileTransferProtocol):
     def _put_ssh_key(self, keyfile_path):
         """Upload an SSH Key to a target"""
         regex = re.compile(
-            r"""ssh-rsa # Only match RSA Keys
+            r"""ssh-(rsa|ed25519)
             \s+(?P<key>[a-zA-Z0-9/+=]+) # Match Keystring
             \s+(?P<comment>.*) # Match comment""", re.X
         )
@@ -319,15 +321,19 @@ class ShellDriver(CommandMixin, Driver, CommandProtocol, FileTransferProtocol):
     def _get_xmodem_rx_cmd(self, filename):
         """ Detect which XMODEM receive command can be used on the target, and cache the result. """
         if not self._xmodem_cached_rx_cmd:
-            if self._run('which rx')[0]:
-                # busybox rx
-                self._xmodem_cached_rx_cmd = "rx '{filename}'"
-            elif self._run('which lrz')[0]:
+            if self._run('which lrz')[2] == 0:
                 # redirect stderr to prevent lrz from printing "ready to receive
                 # $file", which will confuse the XMODEM instance
                 self._xmodem_cached_rx_cmd = "lrz -X -y -c -b '{filename}' 2>/dev/null"
+            elif self._run('which rz')[2] == 0:
+                # renamed binaries packaged by some distros
+                self._xmodem_cached_rx_cmd = "rz -X -y -c -b '{filename}' 2>/dev/null"
+            elif self._run('which rx')[2] == 0:
+                # busybox rx
+                # lrz may provide rx so redirect stderr for the same reason as above
+                self._xmodem_cached_rx_cmd = "rx '{filename}' 2>/dev/null"
             else:
-                raise ExecutionError('No XMODEM receiver (rx, lrz) available on target')
+                raise ExecutionError('No XMODEM receiver (lrz, rz, rx) available on target')
 
         # use the cached string template to make the full command with parameters
         return self._xmodem_cached_rx_cmd.format(filename=filename)
@@ -335,12 +341,15 @@ class ShellDriver(CommandMixin, Driver, CommandProtocol, FileTransferProtocol):
     def _get_xmodem_sx_cmd(self, filename):
         """ Detect which XMODEM send command can be used on the target, and cache the result. """
         if not self._xmodem_cached_sx_cmd:
-            if self._run('which lsz')[0]:
+            if self._run('which lsz')[2] == 0:
                 # redirect stderr to prevent lsz from printing "Give XMODEM receive
                 # cmd now", which will confuse the XMODEM instance
                 self._xmodem_cached_sx_cmd = "lsz -b -X -m 1200 -M 10 '{filename}' 2>/dev/null"
+            elif self._run('which sz')[2] == 0:
+                # renamed binaries packaged by some distros
+                self._xmodem_cached_sx_cmd = "sz -b -X -m 1200 -M 10 '{filename}' 2>/dev/null"
             else:
-                raise ExecutionError('No XMODEM sender (lsz) available on target')
+                raise ExecutionError('No XMODEM sender (lsz, sz) available on target')
 
         # use the cached string template to make the full command with parameters
         return self._xmodem_cached_sx_cmd.format(filename=filename)
