@@ -42,10 +42,13 @@ class SmallUBootDriver(UBootDriver):
 
     Args:
         boot_secret (str): optional, secret used to unlock prompt
+        boot_secret_nolf (bool): optional, send boot_secret without new line
         login_timeout (int): optional, timeout for login prompt detection,
     """
 
+    boot_expression = attr.ib(default=r"U-Boot 20\d+", validator=attr.validators.instance_of(str))
     boot_secret = attr.ib(default="a", validator=attr.validators.instance_of(str))
+    boot_secret_nolf = attr.ib(default=False, validator=attr.validators.instance_of(bool))
     login_timeout = attr.ib(default=60, validator=attr.validators.instance_of(int))
 
     @step()
@@ -57,15 +60,18 @@ class SmallUBootDriver(UBootDriver):
 
         # wait for boot expression. Afterwards enter secret
         self.console.expect(self.boot_expression, timeout=self.login_timeout)
-        self.console.sendline(self.boot_secret)
+        if self.boot_secret_nolf:
+            self.console.write(self.boot_secret.encode('ASCII'))
+        else:
+            self.console.sendline(self.boot_secret)
         self._status = 1
 
         # wait until UBoot has reached it's prompt
         self.console.expect(self.prompt)
-        for command in self.init_commands:  #pylint: disable=not-an-iterable
+        for command in self.init_commands:
             self._run(command)
 
-    def _run(self, cmd: str, *, timeout: int = 30, codec: str = "utf-8", decodeerrors: str = "strict"):  # pylint: disable=unused-argument,line-too-long
+    def _run(self, cmd: str, *, timeout: int = 30, codec: str = "utf-8", decodeerrors: str = "strict"):  # pylint: disable=line-too-long
         """
         If Uboot is in Command-Line mode: Run command cmd and return it's
         output.
