@@ -1,9 +1,10 @@
-# pylint: disable=no-member
 """The SSHDriver uses SSH as a transport to implement CommandProtocol and FileTransferProtocol"""
 import contextlib
 import logging
 import os
+import re
 import stat
+import shlex
 import shutil
 import subprocess
 import tempfile
@@ -20,6 +21,7 @@ from .exception import ExecutionError
 from ..util.helper import get_free_port
 from ..util.proxy import proxymanager
 from ..util.timeout import Timeout
+from ..util.ssh import get_ssh_connect_timeout
 
 
 @target_factory.reg_driver
@@ -30,6 +32,8 @@ class SSHDriver(CommandMixin, Driver, CommandProtocol, FileTransferProtocol):
     priorities = {CommandProtocol: 10, FileTransferProtocol: 10}
     keyfile = attr.ib(default="", validator=attr.validators.instance_of(str))
     stderr_merge = attr.ib(default=False, validator=attr.validators.instance_of(bool))
+    connection_timeout = attr.ib(default=float(get_ssh_connect_timeout()), validator=attr.validators.instance_of(float))
+    explicit_sftp_mode = attr.ib(default=False, validator=attr.validators.instance_of(bool))
 
     def __attrs_post_init__(self):
         super().__attrs_post_init__()
@@ -60,10 +64,15 @@ class SSHDriver(CommandMixin, Driver, CommandProtocol, FileTransferProtocol):
         finally:
             self._cleanup_own_master()
 
+    @property
+    def skip_deactivate_on_export(self):
+        # We need to keep the connection to the target open.
+        return True
+
     def _start_own_master(self):
         """Starts a controlmaster connection in a temporary directory."""
 
-        timeout = Timeout(30.0)
+        timeout = Timeout(self.connection_timeout)
 
         # Retry start of controlmaster, to allow handle failures such as
         # connection refused during target startup
@@ -81,7 +90,7 @@ class SSHDriver(CommandMixin, Driver, CommandProtocol, FileTransferProtocol):
 
     def _start_own_master_once(self, timeout):
 
-        self.tmpdir = tempfile.mkdtemp(prefix='labgrid-ssh-tmp-')
+        self.tmpdir = tempfile.mkdtemp(prefix='lg-ssh-')
         control = os.path.join(
             self.tmpdir, f'control-{self.networkservice.address}'
         )
@@ -117,7 +126,7 @@ class SSHDriver(CommandMixin, Driver, CommandProtocol, FileTransferProtocol):
             #openssh<8.4 requires the DISPLAY var and a detached process with start_new_session=True
             env = {'SSH_ASKPASS': pass_file, 'DISPLAY':'', 'SSH_ASKPASS_REQUIRE':'force'}
             with open(fd, 'w') as f:
-                f.write("#!/bin/sh\necho " + self.networkservice.password)
+                f.write("#!/bin/sh\necho " + shlex.quote(self.networkservice.password))
 
         self.process = subprocess.Popen(args, env=env,
                                         stdout=subprocess.PIPE,
@@ -129,12 +138,14 @@ class SSHDriver(CommandMixin, Driver, CommandProtocol, FileTransferProtocol):
             subprocess_timeout = timeout + 5
             return_value = self.process.wait(timeout=subprocess_timeout)
             if return_value != 0:
-                stdout = self.process.stdout.readlines()
+                stdout, _ = self.process.communicate(timeout=subprocess_timeout)
+                stdout = stdout.split(b"\n")
                 for line in stdout:
                     self.logger.warning("ssh: %s", line.rstrip().decode(encoding="utf-8", errors="replace"))
 
                 try:
-                    proxy_error = open(self.tmpdir+'/proxy-stderr').read().strip()
+                    with open(f'{self.tmpdir}/proxy-stderr') as proxy_err_fd:
+                        proxy_error = proxy_err_fd.read().strip()
                     if proxy_error:
                         raise ExecutionError(
                             f"Failed to connect to {self.networkservice.address} with {' '.join(args)}: error from SSH ProxyCommand: {proxy_error}",  # pylint: disable=line-too-long
@@ -166,10 +177,10 @@ class SSHDriver(CommandMixin, Driver, CommandProtocol, FileTransferProtocol):
 
     @Driver.check_active
     @step(args=['cmd'], result=True)
-    def run(self, cmd, codec="utf-8", decodeerrors="strict", timeout=None): # pylint: disable=unused-argument
-        return self._run(cmd, codec=codec, decodeerrors=decodeerrors)
+    def run(self, cmd, codec="utf-8", decodeerrors="strict", timeout=None):
+        return self._run(cmd, codec=codec, decodeerrors=decodeerrors, timeout=timeout)
 
-    def _run(self, cmd, codec="utf-8", decodeerrors="strict", timeout=None): # pylint: disable=unused-argument
+    def _run(self, cmd, codec="utf-8", decodeerrors="strict", timeout=None):
         """Execute `cmd` on the target.
 
         This method runs the specified `cmd` as a command on its target.
@@ -388,12 +399,28 @@ class SSHDriver(CommandMixin, Driver, CommandProtocol, FileTransferProtocol):
         """The SSHDriver is always connected, return 1"""
         return 1
 
+    def _scp_supports_explicit_sftp_mode(self):
+        version = subprocess.run(["ssh", "-V"], capture_output=True, text=True)
+        version = re.match(r"^OpenSSH_(\d+)\.(\d+)", version.stderr)
+        major, minor = map(int, version.groups())
+
+        # OpenSSH >= 8.6 supports explicitly using the SFTP protocol via -s
+        if major == 8 and minor >= 6:
+            return True
+        # OpenSSH >= 9.0 default to the SFTP protocol
+        if major >= 9:
+            return False
+        raise Exception(f"OpenSSH version {major}.{minor} does not support explicit SFTP mode")
+
     @Driver.check_active
     @step(args=['filename', 'remotepath'])
     def put(self, filename, remotepath=''):
+        ssh_prefix = self.ssh_prefix
+        if self.explicit_sftp_mode and self._scp_supports_explicit_sftp_mode():
+            ssh_prefix.append("-s")
         transfer_cmd = [
             "scp",
-            *self.ssh_prefix,
+            *ssh_prefix,
             "-P", str(self.networkservice.port),
             "-r",
             filename,
@@ -416,9 +443,12 @@ class SSHDriver(CommandMixin, Driver, CommandProtocol, FileTransferProtocol):
     @Driver.check_active
     @step(args=['filename', 'destination'])
     def get(self, filename, destination="."):
+        ssh_prefix = self.ssh_prefix
+        if self.explicit_sftp_mode and self._scp_supports_explicit_sftp_mode():
+            ssh_prefix.append("-s")
         transfer_cmd = [
             "scp",
-            *self.ssh_prefix,
+            *ssh_prefix,
             "-P", str(self.networkservice.port),
             "-r",
             f"{self.networkservice.username}@{self.networkservice.address}:{filename}",
@@ -440,7 +470,7 @@ class SSHDriver(CommandMixin, Driver, CommandProtocol, FileTransferProtocol):
 
     def _cleanup_own_master(self):
         """Exit the controlmaster and delete the tmpdir"""
-        complete_cmd = f"ssh -x -o ControlPath={self.control} -O exit -p {self.networkservice.port} -l {self.networkservice.username} {self.networkservice.address}".split(' ')  # pylint: disable=line-too-long
+        complete_cmd = f"ssh -x -o ControlPath={self.control.replace('%', '%%')} -O exit -p {self.networkservice.port} -l {self.networkservice.username} {self.networkservice.address}".split(' ')  # pylint: disable=line-too-long
         res = subprocess.call(
             complete_cmd,
             stdin=subprocess.DEVNULL,
@@ -452,6 +482,8 @@ class SSHDriver(CommandMixin, Driver, CommandProtocol, FileTransferProtocol):
             self.logger.info("Socket already closed")
         shutil.rmtree(self.tmpdir)
 
+        self.process.communicate()
+
     def _start_keepalive(self):
         """Starts a keepalive connection via the own or external master."""
         args = ["ssh", *self.ssh_prefix, self.networkservice.address, "cat"]
