@@ -1,4 +1,3 @@
-# pylint: disable=unsubscriptable-object
 import socket
 import time
 import enum
@@ -118,14 +117,19 @@ class ResourceMatch:
 
     def ismatch(self, resource_path):
         """Return True if this matches the given resource"""
-        exporter, group, cls, name = resource_path
+        try:
+            exporter, group, cls, name = resource_path
+        except ValueError:
+            exporter, group, cls = resource_path
+            name = None
+
         if not fnmatchcase(exporter, self.exporter):
             return False
         if not fnmatchcase(group, self.group):
             return False
         if not fnmatchcase(cls, self.cls):
             return False
-        if self.name and not fnmatchcase(name, self.name):
+        if name and self.name and not fnmatchcase(name, self.name):
             return False
 
         return True
@@ -148,7 +152,7 @@ class Place:
     def asdict(self):
         # in the coordinator, we have resource objects, otherwise just a path
         acquired_resources = []
-        for resource in self.acquired_resources:  # pylint: disable=not-an-iterable
+        for resource in self.acquired_resources:
             if isinstance(resource, (tuple, list)):
                 acquired_resources.append(resource)
             else:
@@ -186,12 +190,12 @@ class Place:
         if self.tags:
             print(indent + f"tags: {', '.join(k + '=' + v for k, v in sorted(self.tags.items()))}")
         print(indent + "matches:")
-        for match in sorted(self.matches):  # pylint: disable=not-an-iterable
+        for match in sorted(self.matches):
             print(indent + f"  {match}")
         print(indent + f"acquired: {self.acquired}")
         print(indent + "acquired resources:")
         # in the coordinator, we have resource objects, otherwise just a path
-        for resource in sorted(self.acquired_resources):  # pylint: disable=not-an-iterable
+        for resource in sorted(self.acquired_resources):
             if isinstance(resource, (tuple, list)):
                 resource_path = resource
             else:
@@ -213,7 +217,7 @@ class Place:
 
         A resource_path has the structure (exporter, group, cls, name).
         """
-        for match in self.matches:  # pylint: disable=not-an-iterable
+        for match in self.matches:
             if match.ismatch(resource_path):
                 return match
 
@@ -291,8 +295,8 @@ class Reservation:
         if self.prio:
             print(indent + f"prio: {self.prio}")
         print(indent + "filters:")
-        for name, filter in self.filters.items():
-            print(indent + f"  {name}: {' '.join([(k + '=' + v) for k, v in filter.items()])}")
+        for name, fltr in self.filters.items():
+            print(indent + f"  {name}: {' '.join([(k + '=' + v) for k, v in fltr.items()])}")
         if self.allocations:
             print(indent + "allocations:")
             for name, allocation in self.allocations.items():
