@@ -1,6 +1,7 @@
 """The coordinator module coordinates exported resources and clients accessing them."""
 # pylint: disable=no-member,unused-argument
 import asyncio
+import sys
 import traceback
 from collections import defaultdict
 from os import environ
@@ -9,14 +10,13 @@ from enum import Enum
 from functools import wraps
 
 import attr
-import yaml
 from autobahn import wamp
 from autobahn.asyncio.wamp import ApplicationRunner, ApplicationSession
 from autobahn.wamp.types import RegisterOptions
 
-from .common import *
+from .common import *  # pylint: disable=wildcard-import
 from .scheduler import TagSet, schedule
-from ..util import atomic_replace
+from ..util import atomic_replace, yaml
 
 
 class Action(Enum):
@@ -131,7 +131,12 @@ class CoordinatorComponent(ApplicationSession):
         self.save_later()
 
         enable_tcp_nodelay(self)
-        self.join(self.config.realm, ["ticket"], "coordinator")
+        self.join(
+            self.config.realm,
+            authmethods=["anonymous", "ticket"],
+            authid="coordinator",
+            authextra={"authid": "coordinator"},
+        )
 
     def onChallenge(self, challenge):
         return "dummy-ticket"
@@ -194,6 +199,11 @@ class CoordinatorComponent(ApplicationSession):
             'org.labgrid.coordinator.release_place',
             options=RegisterOptions(details_arg='details')
         )
+        await self.register(
+            self.release_place_from,
+            'org.labgrid.coordinator.release_place_from',
+            options=RegisterOptions(details_arg='details')
+        )
         await self.register(
             self.allow_place,
             'org.labgrid.coordinator.allow_place',
@@ -292,10 +302,10 @@ class CoordinatorComponent(ApplicationSession):
         self.save_scheduled = False
 
         resources = self._get_resources()
-        resources = yaml.dump(resources, default_flow_style=False)
+        resources = yaml.dump(resources)
         resources = resources.encode()
         places = self._get_places()
-        places = yaml.dump(places, default_flow_style=False)
+        places = yaml.dump(places)
         places = places.encode()
 
         loop = asyncio.get_event_loop()
@@ -379,7 +389,7 @@ class CoordinatorComponent(ApplicationSession):
         print('join')
         pprint(session_details)
         session = session_details['session']
-        authid = session_details['authid']
+        authid = session_details['authextra'].get('authid') or session_details['authid']
         if authid.startswith('client/'):
             session = ClientSession(self, session, authid)
         elif authid.startswith('exporter/'):
@@ -399,7 +409,7 @@ class CoordinatorComponent(ApplicationSession):
             for groupname, group in session.groups.items():
                 for resourcename in group.copy():
                     action, resource = session.set_resource(groupname, resourcename, {})
-                    await self._update_acquired_places(action, resource, callback=False)  # pylint: disable=not-an-iterable
+                    await self._update_acquired_places(action, resource, callback=False)
         self.save_later()
 
     @locked
@@ -430,7 +440,7 @@ class CoordinatorComponent(ApplicationSession):
                 self._add_default_place(groupname)
         if action in (Action.ADD, Action.DEL):
             async with self.lock:
-                await self._update_acquired_places(action, resource)  # pylint: disable=not-an-iterable
+                await self._update_acquired_places(action, resource)
         self.save_later()
 
     def _get_resources(self):
@@ -583,7 +593,7 @@ class CoordinatorComponent(ApplicationSession):
                                 resource.path[1], resource.path[3], place.name)
                 acquired.append(resource)
         except:
-            print(f"failed to acquire {resource}")
+            print(f"failed to acquire {resource}", file=sys.stderr)
             # cleanup
             await self._release_resources(place, acquired)
             return False
@@ -610,7 +620,7 @@ class CoordinatorComponent(ApplicationSession):
                     await self.call(f'org.labgrid.exporter.{resource.path[0]}.release',
                                     resource.path[1], resource.path[3])
             except:
-                print(f"failed to release {resource}")
+                print(f"failed to release {resource}", file=sys.stderr)
                 # at leaset try to notify the clients
                 try:
                     self._publish_resource(resource)
@@ -650,6 +660,7 @@ class CoordinatorComponent(ApplicationSession):
         self._publish_place(place)
         self.save_later()
         self.schedule_reservations()
+        print(f"{place.name}: place acquired by {place.acquired}")
         return True
 
     @locked
@@ -664,6 +675,43 @@ class CoordinatorComponent(ApplicationSession):
 
         await self._release_resources(place, place.acquired_resources)
 
+        place.acquired = None
+        place.allowed = set()
+        place.touch()
+        self._publish_place(place)
+        self.save_later()
+        self.schedule_reservations()
+        print(f"{place.name}: place released")
+        return True
+
+    @locked
+    async def release_place_from(self, name, acquired, details=None):
+        """
+        Release a place, but only if acquired by a specific user
+
+        Note that unlike the release_place API, this function returns True as
+        long as the specific place is not acquired by the specified user. This
+        may mean that the place was not acquired at all, is acquired by
+        another, or was released; which of these states cannot be inferred from
+        the return code. This is intentional as the purpose of the command is
+        to validate that the specified user no longer owns the place, and the
+        exact state is irrelevant as long as that condition is met.
+
+        Returns:
+            bool: True if the user no longer owns the place, or False if there
+            was an error that prevented releasing the place
+        """
+        try:
+            place = self.places[name]
+        except KeyError:
+            return False
+        if not place.acquired:
+            return True
+        if place.acquired != acquired:
+            return True
+
+        await self._release_resources(place, place.acquired_resources)
+
         place.acquired = None
         place.allowed = set()
         place.touch()
@@ -847,7 +895,7 @@ class CoordinatorComponent(ApplicationSession):
 
 if __name__ == '__main__':
     runner = ApplicationRunner(
-        url=environ.get("WS", u"ws://127.0.0.1:20408/ws"),
+        url=environ.get("WS", "ws://127.0.0.1:20408/ws"),
         realm="realm1",
     )
     runner.run(CoordinatorComponent)
