@@ -1,6 +1,5 @@
 """The remote.exporter module exports resources to the coordinator and makes
 them available to other clients on the same coordinator"""
-# pylint: disable=unsupported-assignment-operation
 import argparse
 import asyncio
 import logging
@@ -112,7 +111,7 @@ class ResourceExport(ResourceEntry):
         start_params = self._get_start_params()
         try:
             self._start(start_params)
-        except Exception:  # pylint: disable=broad-except
+        except Exception:
             self.broken = "start failed"
             self.logger.exception("failed to start with %s", start_params)
             raise
@@ -122,7 +121,7 @@ class ResourceExport(ResourceEntry):
         assert not self.broken
         try:
             self._stop(self.start_params)
-        except Exception:  # pylint: disable=broad-except
+        except Exception:
             self.broken = "stop failed"
             self.logger.exception("failed to stop with %s", self.start_params)
             raise
@@ -159,18 +158,18 @@ class ResourceExport(ResourceEntry):
         if self.broken:
             params['extra']['broken'] = self.broken
         if self.params != params:
-            self.data['params'].update(params)  # pylint: disable=unsubscriptable-object
+            self.data['params'].update(params)
             dirty = True
 
         return dirty
 
-    def acquire(self, *args, **kwargs):  # pylint: disable=arguments-differ
+    def acquire(self, *args, **kwargs):
         if self.broken:
             raise BrokenResourceError(f"cannot acquire broken resource (original reason): {self.broken}")
         super().acquire(*args, **kwargs)
         self.poll()
 
-    def release(self, *args, **kwargs):  # pylint: disable=arguments-differ
+    def release(self, *args, **kwargs):
         if self.broken:
             raise BrokenResourceError(f"cannot release broken resource (original reason): {self.broken}")
         super().release(*args, **kwargs)
@@ -238,6 +237,8 @@ class SerialPortExport(ResourceExport):
                 '-n',
                 '-Y', f'connection: &con01#  accepter: telnet(rfc2217,mode=server),{self.port}',
                 '-Y', f'  connector: serialdev(nouucplock=true),{start_params["path"]},{self.local.speed}n81,local',  # pylint: disable=line-too-long
+                '-Y', '  options:',
+                '-Y', '    max-connections: 10',
             ]
         else:
             cmd = [
@@ -280,26 +281,35 @@ exports["USBSerialPort"] = SerialPortExport
 exports["RawSerialPort"] = SerialPortExport
 
 @attr.s(eq=False)
-class USBNetworkExport(ResourceExport):
-    """ResourceExport for a USB network interface"""
+class NetworkInterfaceExport(ResourceExport):
+    """ResourceExport for a network interface"""
 
     def __attrs_post_init__(self):
         super().__attrs_post_init__()
-        from ..resource.udev import USBNetworkInterface
+        if self.cls == "NetworkInterface":
+            from ..resource.base import NetworkInterface
+            self.local = NetworkInterface(target=None, name=None, **self.local_params)
+        elif self.cls == "USBNetworkInterface":
+            from ..resource.udev import USBNetworkInterface
+            self.local = USBNetworkInterface(target=None, name=None, **self.local_params)
         self.data['cls'] = "RemoteNetworkInterface"
-        self.local = USBNetworkInterface(target=None, name=None, **self.local_params)
 
     def _get_params(self):
         """Helper function to return parameters"""
-        return {
+        params = {
             'host': self.host,
             'ifname': self.local.ifname,
-            'extra': {
+        }
+        if self.cls == "USBNetworkInterface":
+            params['extra'] = {
                 'state': self.local.if_state,
             }
-        }
 
-exports["USBNetworkInterface"] = USBNetworkExport
+        return params
+
+
+exports["USBNetworkInterface"] = NetworkInterfaceExport
+exports["NetworkInterface"] = NetworkInterfaceExport
 
 @attr.s(eq=False)
 class USBGenericExport(ResourceExport):
@@ -489,7 +499,15 @@ class USBFlashableExport(USBGenericExport):
         p['devnode'] = self.local.devnode
         return p
 
+@attr.s(eq=False)
+class USBGenericRemoteExport(USBGenericExport):
+    def __attrs_post_init__(self):
+        super().__attrs_post_init__()
+        self.data['cls'] = f"Remote{self.cls}".replace("Network", "")
+
 exports["AndroidFastboot"] = USBGenericExport
+exports["AndroidUSBFastboot"] = USBGenericRemoteExport
+exports["DFUDevice"] = USBGenericExport
 exports["IMXUSBLoader"] = USBGenericExport
 exports["MXSUSBLoader"] = USBGenericExport
 exports["RKUSBLoader"] = USBGenericExport
@@ -557,7 +575,7 @@ exports["SNMPEthernetPort"] = EthernetPortExport
 
 
 @attr.s(eq=False)
-class GPIOGenericExport(ResourceExport):
+class GPIOSysFSExport(ResourceExport):
     _gpio_sysfs_path_prefix = '/sys/class/gpio'
 
     """ResourceExport for GPIO lines accessed directly from userspace"""
@@ -569,7 +587,7 @@ class GPIOGenericExport(ResourceExport):
         from ..resource import base
         local_cls = getattr(base, local_cls_name)
         self.local = local_cls(target=None, name=None, **self.local_params)
-        self.export_path = Path(GpioGenericExport._gpio_sysfs_path_prefix,
+        self.export_path = Path(GPIOSysFSExport._gpio_sysfs_path_prefix,
                                 f'gpio{self.local.index}')
         self.system_exported = False
 
@@ -593,7 +611,7 @@ class GPIOGenericExport(ResourceExport):
             self.system_exported = True
             return
 
-        export_sysfs_path = os.path.join(GpioGenericExport._gpio_sysfs_path_prefix, 'export')
+        export_sysfs_path = os.path.join(GPIOSysFSExport._gpio_sysfs_path_prefix, 'export')
         with open(export_sysfs_path, mode='wb') as export:
             export.write(str(index).encode('utf-8'))
 
@@ -604,11 +622,11 @@ class GPIOGenericExport(ResourceExport):
         if self.system_exported:
             return
 
-        export_sysfs_path = os.path.join(GpioGenericExport._gpio_sysfs_path_prefix, 'unexport')
+        export_sysfs_path = os.path.join(GPIOSysFSExport._gpio_sysfs_path_prefix, 'unexport')
         with open(export_sysfs_path, mode='wb') as unexport:
             unexport.write(str(index).encode('utf-8'))
 
-exports["SysfsGPIO"] = GPIOGenericExport
+exports["SysfsGPIO"] = GPIOSysFSExport
 
 
 @attr.s
@@ -666,6 +684,21 @@ class LXAIOBusNodeExport(ResourceExport):
 
 exports["LXAIOBusPIO"] = LXAIOBusNodeExport
 
+@attr.s(eq=False)
+class AndroidNetFastbootExport(ResourceExport):
+    def __attrs_post_init__(self):
+        super().__attrs_post_init__()
+        local_cls_name = self.cls
+        self.data['cls'] = f"Remote{self.cls}"
+        from ..resource import fastboot
+        local_cls = getattr(fastboot, local_cls_name)
+        self.local = local_cls(target=None, name=None, **self.local_params)
+
+    def _get_params(self):
+        """Helper function to return parameters"""
+        return {'host' : self.host, **self.local_params}
+
+exports["AndroidNetFastboot"] = AndroidNetFastbootExport
 
 class ExporterSession(ApplicationSession):
     def onConnect(self):
@@ -683,11 +716,17 @@ class ExporterSession(ApplicationSession):
         self.groups = {}
 
         enable_tcp_nodelay(self)
-        self.join(self.config.realm, authmethods=["ticket"], authid=f"exporter/{self.name}")
+        self.join(
+            self.config.realm,
+            authmethods=["anonymous", "ticket"],
+            authid=f"exporter/{self.name}",
+            authextra={"authid": f"exporter/{self.name}"},
+        )
 
     def onChallenge(self, challenge):
         """Function invoked on received challege, returns just a dummy ticket
         at the moment, authentication is not supported yet"""
+        logging.warning("Ticket authentication is deprecated. Please update your coordinator.")
         return "dummy-ticket"
 
     async def onJoin(self, details):
@@ -698,12 +737,20 @@ class ExporterSession(ApplicationSession):
         print(details)
 
         prefix = f'org.labgrid.exporter.{self.name}'
-        await self.register(self.acquire, f'{prefix}.acquire')
-        await self.register(self.release, f'{prefix}.release')
-        await self.register(self.version, f'{prefix}.version')
-
         try:
-            resource_config = ResourceConfig(self.config.extra['resources'])
+            await self.register(self.acquire, f'{prefix}.acquire')
+            await self.register(self.release, f'{prefix}.release')
+            await self.register(self.version, f'{prefix}.version')
+
+            config_template_env = {
+                'env': os.environ,
+                'isolated': self.isolated,
+                'hostname': self.hostname,
+                'name': self.name,
+            }
+            resource_config = ResourceConfig(
+                self.config.extra['resources'], config_template_env
+            )
             for group_name, group in resource_config.data.items():
                 group_name = str(group_name)
                 for resource_name, params in group.items():
@@ -721,7 +768,7 @@ class ExporterSession(ApplicationSession):
                     self.checkpoint = time.monotonic()
 
         except Exception:  # pylint: disable=broad-except
-            traceback.print_exc()
+            traceback.print_exc(file=sys.stderr)
             self.loop.stop()
             return
 
@@ -735,7 +782,7 @@ class ExporterSession(ApplicationSession):
         super().onLeave(details)
 
     async def onDisconnect(self):
-        print("connection lost")
+        print("connection lost", file=sys.stderr)
         global reexec
         reexec = True
         if self.poll_task:
@@ -771,7 +818,7 @@ class ExporterSession(ApplicationSession):
                     changed = resource.poll()
                 except Exception:  # pylint: disable=broad-except
                     print(f"Exception while polling {resource}", file=sys.stderr)
-                    traceback.print_exc()
+                    traceback.print_exc(file=sys.stderr)
                     continue
                 if changed:
                     await self.update_resource(group_name, resource_name)
@@ -787,10 +834,10 @@ class ExporterSession(ApplicationSession):
             except asyncio.CancelledError:
                 break
             except Exception:  # pylint: disable=broad-except
-                traceback.print_exc()
+                traceback.print_exc(file=sys.stderr)
             age = time.monotonic() - self.checkpoint
             if age > 300:
-                print(f"missed checkpoint, exiting (last was {age} seconds ago)")
+                print(f"missed checkpoint, exiting (last was {age} seconds ago)", file=sys.stderr)
                 self.disconnect()
 
     async def add_resource(self, group_name, resource_name, cls, params):
