@@ -1,4 +1,5 @@
-from typing import Dict, Type
+import shlex
+from typing import Dict, Type, List
 import attr
 
 from ..binding import BindingMixin
@@ -29,6 +30,14 @@ class Resource(BindingMixin):
     def command_prefix(self):
         return []
 
+    def wrap_command(self, command):
+        """
+        Returns the command as-is, no need for command wrapping for local resources.
+
+        Must stay compatible with NetworkResource.wrap_command().
+        """
+        return command
+
     @property
     def parent(self):
         return self._parent
@@ -79,6 +88,15 @@ class NetworkResource(Resource):
 
         return prefix + ['--']
 
+    def wrap_command(self, command):
+        """
+        Returns shell-escaped command with prepended command_prefix.
+
+        Must stay compatible with Resource.wrap_command().
+        """
+        shargs = [shlex.quote(a) for a in command]
+        return self.command_prefix + shargs
+
 
 @attr.s(eq=False)
 class ResourceManager:
