@@ -40,16 +40,15 @@ class RemotePlaceManager(ResourceManager):
         # be the same).
         if not self.session:
             self.env = remote_place.target.env
-            config = self.env.config
-            self.url = config.get_option(
-                'crossbar_url',
-                os.environ.get("LG_CROSSBAR", "ws://127.0.0.1:20408/ws"))
-            self.realm = config.get_option(
-                'crossbar_realm',
-                os.environ.get("LG_CROSSBAR_REALM", "realm1"))
+            self.url = os.environ.get("LG_CROSSBAR", "ws://127.0.0.1:20408/ws")
+            self.realm = os.environ.get("LG_CROSSBAR_REALM", "realm1")
+            if self.env:
+                config = self.env.config
+                self.url = config.get_option('crossbar_url', self.url)
+                self.realm = config.get_option('crossbar_realm', self.realm)
             self._start()
-        place = self.session.get_place(remote_place.name)
-        resource_entries = self.session.get_target_resources(place)
+        place = self.session.get_place(remote_place.name)  # pylint: disable=no-member
+        resource_entries = self.session.get_target_resources(place)  # pylint: disable=no-member
         expanded = []
         for resource_name, resource_entry in resource_entries.items():
             new = target_factory.make_resource(
@@ -113,7 +112,33 @@ class RemoteUSBResource(NetworkResource, ManagedResource):
 
 @target_factory.reg_resource
 @attr.s(eq=False)
-class NetworkAndroidFastboot(RemoteUSBResource):
+class RemoteAndroidUSBFastboot(RemoteUSBResource):
+    def __attrs_post_init__(self):
+        self.timeout = 10.0
+        super().__attrs_post_init__()
+
+
+@target_factory.reg_resource
+@attr.s(eq=False)
+class NetworkAndroidFastboot(RemoteAndroidUSBFastboot):
+    def __attrs_post_init__(self):
+        import warnings
+        warnings.warn("NetworkAndroidFastboot is deprecated, use RemoteAndroidUSBFastboot instead",
+                      DeprecationWarning)
+        super().__attrs_post_init__()
+
+
+@target_factory.reg_resource
+@attr.s(eq=False)
+class RemoteAndroidNetFastboot(NetworkResource):
+    address = attr.ib(validator=attr.validators.instance_of(str))
+    port = attr.ib(default=5554, validator=attr.validators.instance_of(int))
+    protocol = attr.ib(default="udp", validator=attr.validators.instance_of(str))
+
+
+@target_factory.reg_resource
+@attr.s(eq=False)
+class NetworkDFUDevice(RemoteUSBResource):
     def __attrs_post_init__(self):
         self.timeout = 10.0
         super().__attrs_post_init__()
