@@ -10,7 +10,8 @@ from .udev import (
     USBTMC,
     USBVideo,
     IMXUSBLoader,
-    AndroidFastboot,
+    AndroidUSBFastboot,
+    DFUDevice,
     USBSDMuxDevice,
     USBSDWireDevice,
     AlteraUSBBlaster,
@@ -21,6 +22,7 @@ from .udev import (
     LXAUSBMux,
     HIDRelay,
     USBDebugger,
+    USBPowerPort,
 )
 from ..util import dump
 
@@ -40,7 +42,8 @@ class Suggester:
         self.resources.append(USBTMC(**args))
         self.resources.append(USBVideo(**args))
         self.resources.append(IMXUSBLoader(**args))
-        self.resources.append(AndroidFastboot(**args))
+        self.resources.append(AndroidUSBFastboot(**args))
+        self.resources.append(DFUDevice(**args))
         self.resources.append(USBMassStorage(**args))
         self.resources.append(USBSDMuxDevice(**args))
         self.resources.append(USBSDWireDevice(**args))
@@ -52,6 +55,7 @@ class Suggester:
         self.resources.append(LXAUSBMux(**args))
         self.resources.append(HIDRelay(**args))
         self.resources.append(USBDebugger(**args))
+        self.resources.append(USBPowerPort(**args, index=0))
 
     def suggest_callback(self, resource, meta, suggestions):
         cls = type(resource).__name__
@@ -78,6 +82,8 @@ class Suggester:
                 dump({cls: {"match": suggestion}}).strip(),
                 '  ',
             ))
+            if cls == 'USBPowerPort':
+                print('    index: ?')
         print("  ---")
         print()
 
