@@ -1,12 +1,11 @@
-# pylint: disable=unsupported-assignment-operation
 import logging
 import os
 import queue
 import warnings
 from collections import OrderedDict
+from importlib import import_module
 
 import attr
-import pyudev
 
 from ..factory import target_factory
 from .common import ManagedResource, ResourceManager
@@ -21,9 +20,10 @@ class UdevManager(ResourceManager):
         self.queue = queue.Queue()
 
         self.log = logging.getLogger('UdevManager')
-        self._context = pyudev.Context()
-        self._monitor = pyudev.Monitor.from_netlink(self._context)
-        self._observer = pyudev.MonitorObserver(self._monitor,
+        self._pyudev = import_module('pyudev')
+        self._context = self._pyudev.Context()
+        self._monitor = self._pyudev.Monitor.from_netlink(self._context)
+        self._observer = self._pyudev.MonitorObserver(self._monitor,
                                                 callback=self._insert_into_queue)
         self._observer.start()
 
@@ -85,17 +85,18 @@ class USBResource(ManagedResource):
         if self.device.properties.get('ID_REVISION'):
             meta['revision'] = self.device.properties.get('ID_REVISION')
 
-        if self.match.get('SUBSYSTEM', None) == 'usb':
-            path = self._get_usb_device().properties.get('ID_PATH')
-            if path:
-                suggestions.append({'ID_PATH': path})
-            serial = self._get_usb_device().properties.get('ID_SERIAL_SHORT')
-            if serial:
-                suggestions.append({'ID_SERIAL_SHORT': serial})
+        path = self.device.properties.get('ID_PATH')
+        if path:
+            suggestions.append({'ID_PATH': path})
         elif self.match.get('@SUBSYSTEM', None) == 'usb':
             path = self._get_usb_device().properties.get('ID_PATH')
             if path:
                 suggestions.append({'@ID_PATH': path})
+
+        serial = self.device.properties.get('ID_SERIAL_SHORT')
+        if serial:
+            suggestions.append({'ID_SERIAL_SHORT': serial})
+        elif self.match.get('@SUBSYSTEM', None) == 'usb':
             serial = self._get_usb_device().properties.get('ID_SERIAL_SHORT')
             if serial:
                 suggestions.append({'@ID_SERIAL_SHORT': serial})
@@ -107,7 +108,7 @@ class USBResource(ManagedResource):
             def match_single(dev, key, value):
                 if dev.properties.get(key) == value:
                     return True
-                if dev.attributes.get(key) == value:
+                if dev.attributes.get(key) and dev.attributes.asstring(key) == value:
                     return True
                 if getattr(dev, key, None) == value:
                     return True
@@ -229,7 +230,7 @@ class USBSerialPort(USBResource, SerialPort):
         if self.port:
             warnings.warn(
                 "USBSerialPort: The port attribute will be overwritten by udev.\n"
-                "Please use udev matching as described in http://labgrid.readthedocs.io/en/latest/getting_started.html#udev-matching"  # pylint: disable=line-too-long
+                "Please use udev matching as described in http://labgrid.readthedocs.io/en/latest/configuration.html#udev-matching"  # pylint: disable=line-too-long
             )
         super().__attrs_post_init__()
 
@@ -310,16 +311,35 @@ class MXSUSBLoader(USBResource):
 
 @target_factory.reg_resource
 @attr.s(eq=False)
-class AndroidFastboot(USBResource):
+class AndroidUSBFastboot(USBResource):
     usb_vendor_id = attr.ib(default='1d6b', validator=attr.validators.instance_of(str))
     usb_product_id = attr.ib(default='0104', validator=attr.validators.instance_of(str))
     def filter_match(self, device):
+        if device.properties.get("adb_user") == "yes":
+            return True
         if device.properties.get('ID_VENDOR_ID') != self.usb_vendor_id:
             return False
         if device.properties.get('ID_MODEL_ID') != self.usb_product_id:
             return False
         return super().filter_match(device)
 
+@target_factory.reg_resource
+@attr.s(eq=False)
+class AndroidFastboot(AndroidUSBFastboot):
+    def __attrs_post_init__(self):
+        warnings.warn("AndroidFastboot is deprecated, use AndroidUSBFastboot instead",
+                      DeprecationWarning)
+        super().__attrs_post_init__()
+
+@target_factory.reg_resource
+@attr.s(eq=False)
+class DFUDevice(USBResource):
+    def filter_match(self, device):
+        if ':fe0102:' not in device.properties.get('ID_USB_INTERFACES', ''):
+            return False
+
+        return super().filter_match(device)
+
 @target_factory.reg_resource
 @attr.s(eq=False)
 class USBNetworkInterface(USBResource, NetworkInterface):
@@ -362,7 +382,7 @@ class SigrokUSBDevice(USBResource):
 
     Args:
         driver (str): driver to use with sigrok
-        channels (str): a sigrok channel mapping as desribed in the sigrok-cli man page
+        channels (str): a sigrok channel mapping as described in the sigrok-cli man page
     """
     driver = attr.ib(
         default=None,
@@ -387,7 +407,7 @@ class SigrokUSBSerialDevice(USBResource):
 
     Args:
         driver (str): driver to use with sigrok
-        channels (str): a sigrok channel mapping as desribed in the sigrok-cli man page
+        channels (str): a sigrok channel mapping as described in the sigrok-cli man page
     """
     driver = attr.ib(
         default=None,
@@ -544,6 +564,14 @@ class SiSPMPowerPort(USBResource):
 @target_factory.reg_resource
 @attr.s(eq=False)
 class USBVideo(USBResource):
+    def filter_match(self, device):
+        capabilities = device.properties.get('ID_V4L_CAPABILITIES').split(':')
+
+        if 'capture' not in capabilities:
+            return False
+
+        return super().filter_match(device)
+
     def __attrs_post_init__(self):
         self.match['SUBSYSTEM'] = 'video4linux'
         self.match['@SUBSYSTEM'] = 'usb'
@@ -658,7 +686,12 @@ class USBDebugger(USBResource):
     def filter_match(self, device):
         match = (device.properties.get('ID_VENDOR_ID'), device.properties.get('ID_MODEL_ID'))
 
-        if match not in [("0403", "6010"),
+        if match not in [("0403", "6010"),  # FT2232C/D/H Dual UART/FIFO IC
+                         ("0483", "374f"),  # STLINK-V3
+                         ("15ba", "0003"),  # Olimex ARM-USB-OCD
+                         ("15ba", "002b"),  # Olimex ARM-USB-OCD-H
+                         ("15ba", "0004"),  # Olimex ARM-USB-TINY
+                         ("15ba", "002a"),  # Olimex ARM-USB-TINY-H
                          ]:
             return False
 
