@@ -2,23 +2,30 @@ from .step import steps
 
 
 class StepReporter:
-    instance = None
+    _started = False
+
+    def __init__(self):
+        from warnings import warn
+
+        warn(
+            "StepReporter should not be instantiated, use StepReporter.start()/.stop() instead.",
+            DeprecationWarning,
+            stacklevel=2,
+        )
 
     @classmethod
     def start(cls):
         """starts the StepReporter"""
-        assert cls.instance is None
-        cls.instance = cls()
+        assert not cls._started
+        steps.subscribe(cls.notify)
+        cls._started = True
 
     @classmethod
     def stop(cls):
         """stops the StepReporter"""
-        assert cls.instance is not None
+        assert cls._started
         steps.unsubscribe(cls.notify)
-        cls.instance = None
-
-    def __init__(self):
-        steps.subscribe(StepReporter.notify)
+        cls._started = False
 
     @staticmethod
     def notify(event):
