@@ -41,18 +41,19 @@ class UBootStrategy(Strategy):
             self.target.activate(self.power)
             self.power.off()
         elif status == Status.uboot:
-            self.transition(Status.off)  # pylint: disable=missing-kwoa
+            self.transition(Status.off)
             self.target.activate(self.console)
             # cycle power
             self.power.cycle()
             # interrupt uboot
             self.target.activate(self.uboot)
         elif status == Status.shell:
-            # tansition to uboot
+            # transition to uboot
             self.transition(Status.uboot)
             self.uboot.boot("")
             self.uboot.await_boot()
             self.target.activate(self.shell)
+            self.shell.run("systemctl is-system-running --wait")
         else:
             raise StrategyError(f"no transition found from {self.status} to {status}")
         self.status = status
