@@ -1,13 +1,14 @@
 import abc
 import atexit
 import logging
+import sys
 from time import monotonic, sleep
 
 import attr
 
 from .binding import BindingError, BindingState
 from .driver import Driver
-from .exceptions import NoSupplierFoundError, NoDriverFoundError, NoResourceFoundError
+from .exceptions import NoSupplierFoundError, NoDriverFoundError, NoResourceFoundError, NoStrategyFoundError
 from .resource import Resource
 from .strategy import Strategy
 from .util import Timeout
@@ -51,8 +52,15 @@ class Target:
         for resource in self.resources:
             resource.poll()
             if not resource.avail and resource.state is BindingState.active:
-                self.log.info("deactivating unavailable resource %s", resource.display_name)  # pylint: disable=line-too-long
-                self.deactivate(resource)
+                deactivated = self.deactivate(resource)
+                deactivated.remove(resource)
+                if deactivated:
+                    self.log.info("deactivating unavailable resource %s used by %s",
+                        resource.display_name,
+                        ", ".join(d.display_name for d in deactivated)
+                    )
+                else:
+                    self.log.debug("deactivating unavailable resource %s (unused)", resource.display_name)  # pylint: disable=line-too-long
 
     def await_resources(self, resources, timeout=None, avail=True):
         """
@@ -109,16 +117,25 @@ class Target:
         """
         found = []
         other_names = []
+        default = None
         if isinstance(cls, str):
             cls = target_factory.class_from_string(cls)
 
         for res in self.resources:
             if not isinstance(res, cls):
                 continue
+            if res.name == "default":
+                default = res
             if name and res.name != name:
                 other_names.append(res.name)
                 continue
             found.append(res)
+
+        # if no explicit resource name is requested and a "default" resource was saved and
+        # multiple resources were found, use the default resource
+        if not name and default and len(found) != 1:
+            found = [default]
+
         if not found:
             name_msg = f" named '{name}'" if name else ""
             if other_names:
@@ -131,7 +148,7 @@ class Target:
             )
         elif len(found) > 1:
             raise NoResourceFoundError(
-                f"multiple resources matching {cls} found in {self}"
+                f"multiple resources matching {cls} found in {self}", found=found
             )
         if wait_avail:
             self.await_resources(found)
@@ -212,18 +229,39 @@ class Target:
         """
         return self._get_driver(cls, name=name, activate=activate)
 
+    def get_strategy(self):
+        """
+        Helper function to get the strategy of the target.
+
+        Returns the Strategy, if exactly one exists and raises a
+        NoStrategyFoundError otherwise.
+        """
+        found = []
+        for drv in self.drivers:
+            if not isinstance(drv, Strategy):
+                continue
+            found.append(drv)
+        if not found:
+            raise NoStrategyFoundError(f"no Strategy found in {self}")
+        elif len(found) > 1:
+            raise NoStrategyFoundError(f"multiple Strategies found in {self}")
+        return found[0]
+
     def __getitem__(self, key):
         """
         Syntactic sugar to access drivers by class (optionally filtered by
         name).
 
+        >>> from labgrid import Target
+        >>> from labgrid.driver.fake import FakeConsoleDriver
+        >>>
         >>> target = Target('main')
         >>> console = FakeConsoleDriver(target, 'console')
         >>> target.activate(console)
         >>> target[FakeConsoleDriver]
-        FakeConsoleDriver(target=Target(name='main', …), name='console', …)
+        FakeConsoleDriver(target=Target(name='main', env=None), name='console', state=<BindingState.active: 2>, txdelay=0.0)
         >>> target[FakeConsoleDriver, 'console']
-        FakeConsoleDriver(target=Target(name='main', …), name='console', …)
+        FakeConsoleDriver(target=Target(name='main', env=None), name='console', state=<BindingState.active: 2>, txdelay=0.0)
         """
         name = None
         if not isinstance(key, tuple):
@@ -431,6 +469,8 @@ class Target:
         Recursively deactivate the client's clients and itself.
 
         This is needed to ensure that no client has an inactive supplier.
+
+        Returns the list of all objects that were deactivated
         """
         if isinstance(client, str):
             cls = target_factory.class_from_string(client)
@@ -439,7 +479,7 @@ class Target:
         assert client is not None
 
         if client.state is BindingState.bound:
-            return  # nothing to do
+            return [] # nothing to do
 
         if client.state is not BindingState.active:
             raise BindingError(
@@ -449,12 +489,15 @@ class Target:
         # consistency check
         assert client in self.resources or client in self.drivers
 
+        deactivated = [client]
+
         for cli in client.clients:
-            self.deactivate(cli)
+            deactivated.extend(self.deactivate(cli))
 
         # update state
         client.on_deactivate()
         client.state = BindingState.bound
+        return deactivated
 
     def deactivate_all_drivers(self):
         """Deactivates all drivers in reversed order they were activated"""
@@ -466,11 +509,49 @@ class Target:
             self.cleanup()
         except Exception as e:
             print("An exception occured during cleanup, call the cleanup() "
-                  "method on targets yourself to handle exceptions explictly.")
-            print(f"Error: {e}")
+                  "method on targets yourself to handle exceptions explictly.", file=sys.stderr)
+            print(f"Error: {e}", file=sys.stderr)
+            import traceback
+            traceback.print_exc(file=sys.stderr)
+
+    def export(self):
+        """
+        Export information from drivers.
+
+        All drivers are deactivated before being exported, unless their
+        skip_deactivate_on_export property is true.
+
+        The Strategy can decide for which driver the export method is called and
+        with which name. Otherwise, all drivers are exported.
+        """
+        try:
+            name_map = self.get_strategy().prepare_export()
+            selection = set(name_map.keys())
+        except NoStrategyFoundError:
+            name_map = {}
+            selection = set(driver for driver in self.drivers if not isinstance(driver, Strategy))
+
+        assert len(name_map) == len(set(name_map.values())), "duplicate export name"
+
+        # drivers may need to be deactivated for export to avoid conflicts
+        for drv in reversed(self.drivers):
+            if not drv.skip_deactivate_on_export:
+                self.deactivate(drv)
+
+        export_vars = {}
+        for driver in selection:
+            name = name_map.get(driver)
+            if not name:
+                name = driver.get_export_name()
+            for k, v in driver.get_export_vars().items():
+                assert isinstance(k, str), f"key {k} from {driver} is not a string"
+                assert isinstance(v, str), f"value {v} for key {k} from {driver} is not a string"
+                export_vars[f"LG__{name}_{k}".upper()] = v
+        return export_vars
+
 
     def cleanup(self):
-        """Clean up conntected drivers and resources in reversed order"""
+        """Clean up connected drivers and resources in reversed order"""
         self.deactivate_all_drivers()
         for res in reversed(self.resources):
             self.deactivate(res)
