@@ -27,13 +27,13 @@ class QMPMonitor:
 
     def _read_parse_json(self):
         line = self.monitor_out.readline().decode('utf-8')
-        self.logger.debug("Received line: %s", line)
+        self.logger.debug("Received line: %s", line.rstrip("\r\n"))
         if not line:
             raise QMPError("Received empty response")
         return json.loads(line)
 
-    def execute(self, command):
-        json_command = {"execute": command}
+    def execute(self, command, arguments={}):
+        json_command = {"execute": command, "arguments": arguments}
 
         self.monitor_in.write(json.dumps(json_command).encode("utf-8"))
         self.monitor_in.write("\n".encode("utf-8"))
