@@ -1,4 +1,3 @@
-# pylint: disable=no-member
 import atexit
 import tempfile
 import logging
@@ -12,10 +11,13 @@ from typing import Dict
 import attr
 from ..driver.exception import ExecutionError
 
-from .helper import get_free_port
+from .helper import get_free_port, processwrapper
 
 __all__ = ['sshmanager', 'SSHConnection', 'ForwardError']
 
+def get_ssh_connect_timeout():
+    return int(os.environ.get("LG_SSH_CONNECT_TIMEOUT", 30))
+
 
 @attr.s
 class SSHConnectionManager:
@@ -43,7 +45,6 @@ class SSHConnectionManager:
             :obj:`SSHConnection`: the SSHConnection for the host"""
         instance = self._connections.get(host)
         if instance is None:
-            # pylint: disable=unsupported-assignment-operation
             self.logger.debug("Creating SSHConnection for %s", host)
             instance = SSHConnection(host)
             instance.connect()
@@ -58,20 +59,17 @@ class SSHConnectionManager:
         Arguments:
             connection (:obj:`SSHConnection`): SSHconnection to add to the manager
         """
-        # pylint: disable=unsupported-assignment-operation
         assert isinstance(connection, SSHConnection)
         if connection.host not in self._connections:
             self._connections[connection.host] = connection
 
     def remove_connection(self, connection):
-        # pylint: disable=unsupported-assignment-operation
         assert isinstance(connection, SSHConnection)
         if connection.isconnected():
             raise ExecutionError("Can't remove connected connection")
         del self._connections[connection.host]
 
     def remove_by_name(self, name):
-        # pylint: disable=unsupported-assignment-operation
         del self._connections[name]
 
     def open(self, host):
@@ -117,7 +115,7 @@ def _check_connected(func):
             raise ExecutionError(
                 f"{func.__qualname__} can not be called ({self} is not connected)"
             )
-        return func(self, *_args, **_kwargs)  # pylint: disable=not-callable
+        return func(self, *_args, **_kwargs)
 
     return wrapper
 
@@ -136,11 +134,12 @@ class SSHConnection:
         default=False, init=False, validator=attr.validators.instance_of(bool)
     )
     _tmpdir = attr.ib(
-        default=attr.Factory(lambda: tempfile.mkdtemp(prefix="labgrid-connection-")),
+        default=attr.Factory(lambda: tempfile.mkdtemp(prefix="lg-con-")),
         init=False,
         validator=attr.validators.instance_of(str)
     )
-    _forwards = attr.ib(init=False, default=attr.Factory(dict))
+    _l_forwards = attr.ib(init=False, default=attr.Factory(dict))
+    _r_forwards = attr.ib(init=False, default=attr.Factory(set))
 
     def __attrs_post_init__(self):
         self._logger = logging.getLogger(f"{self}")
@@ -264,7 +263,8 @@ class SSHConnection:
                     if loglevel is not None:
                         self._logger.log(loglevel, line)
 
-        return stdout, stderr, sub.wait()
+        sub.communicate()
+        return stdout, stderr, sub.returncode
 
     def run_check(self, command, *, codec="utf-8", decodeerrors="strict",
                   force_tty=False, stderr_merge=False, stderr_loglevel=None,
@@ -322,51 +322,79 @@ class SSHConnection:
     @_check_connected
     def put_file(self, local_file, remote_path):
         """Put a file onto the remote host"""
-        complete_cmd = ["rsync", "--compress=1", "--sparse", "--copy-links", "--verbose", "--progress", "--times", "-e",
+        complete_cmd = ["rsync", "--compress", "--sparse", "--copy-links", "--verbose", "--progress", "--times", "-e",
                         " ".join(['ssh'] + self._get_ssh_args())]
         complete_cmd += [
             f"{local_file}",
             f"{self.host}:{remote_path}"
         ]
         self._logger.debug("Running command: %s", complete_cmd)
-        subprocess.check_call(
+        processwrapper.check_output(
             complete_cmd,
             stdin=subprocess.DEVNULL,
+            print_on_silent_log=True
         )
 
     @_check_connected
-    def add_port_forward(self, remote_host, remote_port):
+    def add_port_forward(self, remote_host, remote_port, local_port=None):
         """forward command"""
-        local_port = get_free_port()
+        if local_port is None:
+            local_port = get_free_port()
         destination = f"{remote_host}:{remote_port}"
 
-        # pylint: disable=not-an-iterable
-        if destination in self._forwards:
-            return self._forwards[destination]
+        if destination in self._l_forwards:
+            return self._l_forwards[destination]
         self._run_socket_command(
             "forward", [
                 f"-L{local_port}:{destination}"
             ]
         )
-        self._forwards[destination] = local_port
+        self._l_forwards[destination] = local_port
         return local_port
 
     @_check_connected
     def remove_port_forward(self, remote_host, remote_port):
         """cancel command"""
         destination = f"{remote_host}:{remote_port}"
-        local_port = self._forwards.pop(destination, None)
+        local_port = self._l_forwards.pop(destination, None)
 
-        # pylint: disable=not-an-iterable
         if local_port is None:
             raise ForwardError("Forward does not exist")
 
         self._run_socket_command(
             "cancel", [
-                f"-L{local_port}:localhost:{remote_port}"
+                f"-L{local_port}:{destination}"
             ]
         )
 
+    @_check_connected
+    def add_remote_port_forward(self, remote_port, local_port, remote_bind=None):
+        """remote forward command
+
+        Note that the remote socket is not *bound* to any specific IP by
+        default, making it reachable by the target. Also, 'GatewayPorts
+        clientspecified' needs to be configured in the remote host's
+        sshd_config.
+        """
+        if remote_bind is None:
+            remote_bind = "*"
+
+        forward = f"-R{remote_bind}:{remote_port:d}:localhost:{local_port:d}"
+
+        self._run_socket_command("forward", [forward])
+        self._r_forwards.add(forward)
+
+    @_check_connected
+    def remove_remote_port_forward(self, remote_port, local_port, remote_bind=None):
+        """remote cancel command"""
+        if remote_bind is None:
+            remote_bind = "*"
+
+        forward = f"-R{remote_bind}:{remote_port:d}:localhost:{local_port:d}"
+
+        self._r_forwards.remove(forward)
+        self._run_socket_command("cancel", [forward])
+
     def connect(self):
         if not self._connected:
             self._open_connection()
@@ -400,11 +428,13 @@ class SSHConnection:
         """Starts a controlmaster connection in a temporary directory."""
         control = os.path.join(self._tmpdir, f'control-{self.host}')
 
+        connect_timeout = get_ssh_connect_timeout()
+
         self._logger.debug("ControlSocket: %s", control)
         args = ["ssh"] + SSHConnection._get_ssh_base_args()
         args += [
             "-n", "-MN",
-            "-o", "ConnectTimeout=30",
+            "-o", f"ConnectTimeout={connect_timeout}",
             "-o", "ControlPersist=300",
             "-o", "ControlMaster=yes",
             "-o", f"ControlPath={control}",
@@ -423,13 +453,16 @@ class SSHConnection:
         )
 
         try:
-            if self._master.wait(timeout=30) != 0:
+            if self._master.wait(timeout=connect_timeout) != 0:
+                stdout, stderr = self._master.communicate()
                 raise ExecutionError(
-                    f"failed to connect to {self.host} with args {args}, returncode={self._master.wait()} {self._master.stdout.readlines()},{self._master.stderr.readlines()} "  # pylint: disable=line-too-long
+                    f"failed to connect to {self.host} with args {args}, returncode={self._master.returncode} {stdout},{stderr}"  # pylint: disable=line-too-long
                 )
         except subprocess.TimeoutExpired:
+            self._master.kill()
+            stdout, stderr = self._master.communicate()
             raise ExecutionError(
-                f"failed to connect (timeout) to {self.host} with args {args} {self._master.stdout.readlines()},{self._master.stderr.readlines()}"  # pylint: disable=line-too-long
+                f"failed to connect (timeout) to {self.host} with args {args}, process killed, got {stdout},{stderr}"  # pylint: disable=line-too-long
             )
 
         if not os.path.exists(control):
@@ -448,7 +481,7 @@ class SSHConnection:
             self._run_socket_command("exit")
             # if the master doesn't terminate in less than 60 seconds,
             # something is very wrong
-            self._master.wait(timeout=60)
+            self._master.communicate(timeout=60)
         finally:
             self._socket = None
             self._master = None
@@ -463,6 +496,7 @@ class SSHConnection:
             stdin=subprocess.PIPE,
             stdout=subprocess.PIPE,
             stderr=subprocess.PIPE,
+            start_new_session=True,
         )
 
         self._logger.debug('Started keepalive for %s', self.host)
@@ -479,9 +513,7 @@ class SSHConnection:
             self._keepalive.communicate(timeout=60)
         except subprocess.TimeoutExpired:
             self._keepalive.kill()
-
-        try:
-            self._keepalive.wait(timeout=60)
+            self._keepalive.communicate(timeout=60)
         finally:
             self._keepalive = None
 
@@ -498,6 +530,14 @@ class SSHConnection:
 
     def cleanup(self):
         if self.isconnected():
+            # cancel local forwards
+            for destination, local_port in self._l_forwards.items():
+                self._run_socket_command("cancel", [f"-L{local_port}:{destination}"])
+            self._l_forwards.clear()
+            # cancel remote forwards
+            for forward in self._r_forwards:
+                self._run_socket_command("cancel", [forward])
+            self._r_forwards.clear()
             self.disconnect()
         shutil.rmtree(self._tmpdir)
 
