@@ -1,3 +1,5 @@
+from urllib.parse import urlparse
+
 import pytest
 
 from labgrid.resource import NetworkPowerPort
@@ -54,13 +56,16 @@ class TestExternalPowerDriver:
         fdopen = mocker.patch('os.fdopen')
         close = mocker.patch('os.close')
         popen = mocker.patch('subprocess.Popen')
+        select = mocker.patch('select.select')
         fd_mock = mocker.MagicMock()
         instance_mock = mocker.MagicMock()
+        select_mock = mocker.MagicMock()
         popen.return_value = instance_mock
         fdopen.return_value = fd_mock
         pty.return_value = (instance_mock, 2)
         fd_mock.read.return_value = b'Done\nDone'
         instance_mock.returncode = 0
+        select.return_value = ([], [], [])
 
         d = ExternalPowerDriver(
             target, 'power', cmd_on='set -1 foo-board', cmd_off='set -0 foo-board'
@@ -77,13 +82,16 @@ class TestExternalPowerDriver:
         fdopen = mocker.patch('os.fdopen')
         close = mocker.patch('os.close')
         popen = mocker.patch('subprocess.Popen')
+        select = mocker.patch('select.select')
         fd_mock = mocker.MagicMock()
         instance_mock = mocker.MagicMock()
+        select_mock = mocker.MagicMock()
         popen.return_value = instance_mock
         fdopen.return_value = fd_mock
         pty.return_value = (instance_mock, 2)
         fd_mock.read.return_value = b'Done\nDone'
         instance_mock.returncode = 0
+        select.return_value = ([], [], [])
 
         d = ExternalPowerDriver(
             target, 'power', cmd_on='set -1 foo-board', cmd_off='set -0 foo-board'
@@ -100,13 +108,16 @@ class TestExternalPowerDriver:
         fdopen = mocker.patch('os.fdopen')
         close = mocker.patch('os.close')
         popen = mocker.patch('subprocess.Popen')
+        select = mocker.patch('select.select')
         fd_mock = mocker.MagicMock()
         instance_mock = mocker.MagicMock()
+        select_mock = mocker.MagicMock()
         popen.return_value = instance_mock
         fdopen.return_value = fd_mock
         pty.return_value = (instance_mock, 2)
         fd_mock.read.return_value = b'Done\nDone'
         instance_mock.returncode = 0
+        select.return_value = ([], [], [])
 
         m_sleep = mocker.patch('time.sleep')
 
@@ -130,13 +141,16 @@ class TestExternalPowerDriver:
         fdopen = mocker.patch('os.fdopen')
         close = mocker.patch('os.close')
         popen = mocker.patch('subprocess.Popen')
+        select = mocker.patch('select.select')
         fd_mock = mocker.MagicMock()
         instance_mock = mocker.MagicMock()
+        select_mock = mocker.MagicMock()
         popen.return_value = instance_mock
         fdopen.return_value = fd_mock
         pty.return_value = (instance_mock, 2)
         fd_mock.read.return_value = b'Done\nDone'
         instance_mock.returncode = 0
+        select.return_value = ([], [], [])
 
         m_sleep = mocker.patch('time.sleep')
 
@@ -159,16 +173,87 @@ class TestNetworkPowerDriver:
         d = NetworkPowerDriver(target, 'power')
         assert isinstance(d, NetworkPowerDriver)
 
+    @pytest.mark.parametrize('backend', ('rest', 'simplerest'))
+    @pytest.mark.parametrize(
+        'host',
+        (
+            'http://example.com/{index}',
+            'https://example.com/{index}',
+            'http://example.com:1234/{index}',
+            'https://example.com:1234/{index}',
+            'http://user:pass@example.com:1234/{index}',
+            'https://user:pass@example.com:1234/{index}',
+        )
+    )
+    def test_create_backend_with_url_in_host(self, target, mocker, backend, host):
+        get = mocker.patch('requests.get')
+        get.return_value.text = '1'
+        mocker.patch('requests.put')
+
+        index = '1'
+        NetworkPowerPort(target, 'power', model=backend, host=host, index=index)
+        d = NetworkPowerDriver(target, 'power')
+        assert isinstance(d, NetworkPowerDriver)
+        target.activate(d)
+
+        d.cycle()
+        assert d.get() is True
+
+        # the called URL should be similar to the one configured in the resource, but with
+        # index and explicit port
+        expected_host = host.format(index=index)
+        url = urlparse(expected_host)
+        if url.port is None:
+            implicit_port = 443 if url.scheme == 'https' else 80
+            expected_host = expected_host.replace(url.netloc, f'{url.netloc}:{implicit_port}')
+
+        get.assert_called_with(expected_host)
+
+    @pytest.mark.parametrize(
+        'host',
+        (
+            'http://example.com',
+            'https://example.com',
+        )
+    )
+    def test_create_shelly_gen1_backend_with_url_in_host(self, target, mocker, host):
+        get = mocker.patch('requests.get')
+        get.return_value.text = '{"ison": true}'
+        mocker.patch('requests.post')
+
+        index = '0'
+        NetworkPowerPort(target, 'power', model='shelly_gen1', host=host, index=index)
+        d = NetworkPowerDriver(target, 'power')
+        target.activate(d)
+
+        d.cycle()
+        assert d.get() is True
+
+        # the called URL should be similar to the one configured in the resource, but with
+        # index and explicit port
+        expected_host = f"{host}/relay/{index}"
+        url = urlparse(expected_host)
+        if url.port is None:
+            implicit_port = 443 if url.scheme == 'https' else 80
+            expected_host = expected_host.replace(url.netloc, f'{url.netloc}:{implicit_port}')
+
+        get.assert_called_with(expected_host)
+
     def test_import_backends(self):
         import labgrid.driver.power
         import labgrid.driver.power.apc
+        import labgrid.driver.power.eaton
         import labgrid.driver.power.digipower
+        import labgrid.driver.power.digitalloggers_http
         import labgrid.driver.power.gude
         import labgrid.driver.power.gude24
+        import labgrid.driver.power.tplink
         import labgrid.driver.power.netio
         import labgrid.driver.power.netio_kshell
         import labgrid.driver.power.rest
         import labgrid.driver.power.sentry
+        import labgrid.driver.power.eg_pms2_network
+        import labgrid.driver.power.shelly_gen1
 
     def test_import_backend_siglent(self):
         pytest.importorskip("vxi11")
