@@ -9,7 +9,7 @@ from labgrid.binding import BindingError
 from labgrid.resource import Resource
 from labgrid.driver import Driver
 from labgrid.strategy import Strategy
-from labgrid.exceptions import NoSupplierFoundError, NoDriverFoundError, NoResourceFoundError
+from labgrid.exceptions import NoSupplierFoundError, NoDriverFoundError, NoResourceFoundError, NoStrategyFoundError
 
 
 # test basic construction
@@ -27,6 +27,19 @@ def test_get_resource(target):
     assert target.get_resource(A) is a
     assert target.get_resource(A, name="aresource") is a
 
+    # make sure resources named "default" are prioritized
+    b = A(target, "default")
+    assert target.get_resource(A) is b
+    assert target.get_resource(A, name="aresource") is a
+
+def test_get_resource_multiple_no_default(target):
+    class A(Resource):
+        pass
+
+    a = A(target, "aresource")
+    b = A(target, "default")
+    with pytest.raises(NoResourceFoundError) as excinfo:
+        target.get_resource(A, name="nosuchresource")
 
 def test_get_driver(target):
     class A(Driver):
@@ -252,6 +265,27 @@ def test_suppliers_optional_named_a_missing(target):
 
 
 
+class StrategyA(Strategy):
+    bindings = {
+        "drv": DriverWithA,
+    }
+
+
+def test_get_strategy(target):
+    ra = ResourceA(target, "resource")
+    d = DriverWithA(target, "driver")
+
+    with pytest.raises(NoStrategyFoundError):
+        target.get_strategy()
+
+    s1 = StrategyA(target, "s1")
+    assert target.get_strategy() is s1
+
+    s2 = StrategyA(target, "s2")
+    with pytest.raises(NoStrategyFoundError):
+        target.get_strategy()
+
+
 # test nested resource creation
 @attr.s(eq=False)
 class DiscoveryResource(Resource):
